<?php

/**
 * Create temporary table and store data from easy_email entities.
 */
function easy_email_post_update_00001(&$sandbox) {
  $connection = \Drupal::database();
  try {
    $connection->schema()->createTable('easy_email_temp', [
      'description' => 'Temporary table to store data from easy_email entities.',
      'fields' => [
        'id' => [
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
        ],
        'vid' => [
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
        ],
        'data' => [
          'type' => 'blob',
          'not null' => TRUE,
        ]
      ],
    ]);
  }
  catch (\Drupal\Core\Database\SchemaObjectExistsException $e) {
    // This is okay.
  }

  $easy_email_storage = \Drupal::entityTypeManager()->getStorage('easy_email');

  if (!isset($sandbox['total'])) {
    $sandbox['total'] = $easy_email_storage->getQuery()
      ->accessCheck(FALSE)
      ->count()
      ->execute();
    $sandbox['current'] = 0;
  }
  // Loop through the easy_email entities 100 at a time.
  // Store the progress in the sandbox variable.
  if (!isset($sandbox['last_id'])) {
    $sandbox['last_id'] = 0;
  }

  $easy_email_ids = $easy_email_storage->getQuery()
    ->condition('id', $sandbox['last_id'], '>')
    ->accessCheck(FALSE)
    ->sort('id')
    ->range(0, 100)
    ->execute();
  $easy_emails = $easy_email_storage->loadMultiple($easy_email_ids);
  if (count($easy_emails) === 0) {
    $sandbox['#finished'] = 1;
    return;
  }
  else {
    $sandbox['#finished'] = 0;
  }
  foreach ($easy_emails as $easy_email) {
    // Get all the revisions of the entity
    $revision_ids = $easy_email_storage->revisionIds($easy_email);
    // Loop through the revisions and store the data in the temporary table
    foreach ($revision_ids as $revision_id) {
      $revision = $easy_email_storage->loadRevision($revision_id);
      if ($revision !== NULL) {
        $field_values = $revision->toArray();
        $field_values = serialize($field_values);
        $connection->insert('easy_email_temp')
          ->fields([
            'id' => $revision->id(),
            'vid' => $revision->getRevisionId(),
            'data' => $field_values,
          ])
          ->execute();
      }
    }
    $sandbox['last_id'] = $easy_email->id();
    $sandbox['current']++;
  }
  return $sandbox['current'] . ' emails of ' . $sandbox['total'] . ' stored in temp table.';
}

/**
 * Delete all easy_email entities.
 */
function easy_email_post_update_00002(&$sandbox) {
  $easy_email_storage = \Drupal::entityTypeManager()->getStorage('easy_email');

  if (!isset($sandbox['total'])) {
    $sandbox['total'] = $easy_email_storage->getQuery()
      ->accessCheck(FALSE)
      ->count()
      ->execute();
    $sandbox['current'] = 0;
  }
  // Loop through the easy_email entities 100 at a time.
  $easy_email_ids = $easy_email_storage->getQuery()
    ->accessCheck(FALSE)
    ->sort('id')
    ->range(0, 100)
    ->execute();
  $easy_emails = $easy_email_storage->loadMultiple($easy_email_ids);
  if (count($easy_emails) === 0) {
    $sandbox['#finished'] = 1;
    return;
  }
  else {
    $sandbox['#finished'] = 0;
  }
  $easy_email_storage->delete($easy_emails);
  $sandbox['current'] += count($easy_emails);

  return $sandbox['current'] . ' emails of ' . $sandbox['total'] . ' deleted.';
}

/**
 * Remove all easy_email module bundle fields from the easy_email types.
 */
function easy_email_post_update_00003(&$sandbox) {
  $easy_email_bundle_fields = [
    'key',
    'recipient_uid',
    'cc_uid',
    'cc_address',
    'bcc_uid',
    'bcc_address',
    'from_name',
    'from_address',
    'reply_to',
    'body_html',
    'body_plain',
    'inbox_preview',
    'attachment',
    'attachment_path',
  ];
  $easy_email_type_storage = \Drupal::entityTypeManager()->getStorage('easy_email_type');

  $easy_email_type_ids = $easy_email_type_storage->getQuery()
    ->accessCheck(FALSE)
    ->execute();
  foreach ($easy_email_type_ids as $easy_email_type) {
    foreach ($easy_email_bundle_fields as $field_name) {
      $field_config = \Drupal\field\Entity\FieldConfig::loadByName('easy_email', $easy_email_type, $field_name);
      if ($field_config !== NULL) {
        $field_config->delete();
      }
    }
  }

  foreach ($easy_email_bundle_fields as $field_name) {
    $field_storage = \Drupal\field\Entity\FieldStorageConfig::loadByName('easy_email', $field_name);
    if ($field_storage !== NULL) {
      $field_storage->delete();
    }
  }
}

/**
 * Uninstall easy_email entity type
 */
function easy_email_post_update_00004(&$sandbox) {
  $entity_update_manager = \Drupal::entityDefinitionUpdateManager();
  $entity_type = $entity_update_manager->getEntityType('easy_email');
  $entity_update_manager->uninstallEntityType($entity_type);
}


